==> apollopay-1.0.1.tar.gz: lib/src/api/entities/payment/pricing/price_entity.dart <==
import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_tier_entity.dart';
import 'package:apollopay/src/api/entities/payment/product_entity.dart';
import 'package:apollopay/src/api/entities/reccurency.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Price extends Entity {
  String? id;
  String? token;
  String? productToken;
  String? currency;
  int amount;
  String? name;
  String? description;
  bool? active;
  String? unitLabel;
  Map<String, String>? metadata;
  List<String> taxes;
  List<Tier>? tiers;
  Recurrency periodType;
  int? periodLength;
  bool archived;

  // Assigned by the controller
  Product? product;

  double getPrice(double quantity) {
    if (tiers == null || tiers?.isEmpty == true) {
      return quantity * amount;
    }

    double calculatedAmount = amount.toDouble();
    double quantityLeft = quantity;

    tiers?.sort((a, b) {
      return (a.upperLimit ?? double.infinity)
          .compareTo((b.upperLimit ?? double.infinity));
    });

    for (var tier in tiers!) {
      tier.upperLimit ??= double.infinity;
      if (quantity <= (tier.upperLimit ?? 0)) {
        calculatedAmount += tier.amount + (tier.unitAmount * quantityLeft);
        quantityLeft -= tier.upperLimit ?? 0;
      }
    }

    return calculatedAmount;
  }

  Price(
      {required this.amount,
      this.currency,
      this.id,
      this.token,
      this.productToken,
      this.name,
      this.description,
      this.active,
      this.taxes = const <String>[],
      this.unitLabel,
      this.metadata,
      this.tiers,
      this.periodType = Recurrency.onetime,
      this.periodLength,
      this.archived = false,
      this.product});

  @override
  Price copyWith(
      {String? currency,
      int? amount,
      String? name,
      String? description,
      bool? active,
      String? unitLabel,
      Map<String, String>? metadata,
      List<String>? taxesTokens,
      List<Tier>? tiers,
      bool? recurring,
      int? reccurencyPerYear,
      bool? archived,
      Product? product}) {
    return Price(
        id: id,
        token: token,
        productToken: productToken,
        currency: currency ?? this.currency,
        amount: amount ?? this.amount,
        name: name ?? this.name,
        description: description ?? this.description,
        active: active ?? this.active,
        unitLabel: unitLabel ?? this.unitLabel,
        metadata: metadata ?? this.metadata,
        taxes: taxesTokens ?? this.taxes,
        tiers: tiers ?? this.tiers,
        periodType: periodType,
        periodLength: periodLength,
        archived: archived ?? this.archived,
        product: product ?? this.product);
  }

  @override
  Map<String, dynamic> toMap({bool databaseFormat = false}) {
    return {
      if (databaseFormat) '_id': id,
      'product_token': productToken,
      'token': token,
      'currency': currency,
      'amount': amount,
      'name': name,
      'description': description,
      'active': active,
      'unit_label': unitLabel,
      'taxes': taxes,
      'metadata': metadata,
      'tiers': tiers?.map((tier) => tier.toMap()).toList(),
      'period_type': periodType.index,
      'period_length': periodLength,
      'archived': archived,
      if (product != null) 'product': product?.toMap()
    };
  }

  factory Price.fromMap(Map<String, dynamic> map) {
    assert(map['name'] != null, 'PriceError -> name must be set.');
    assert(
        map['amount'] != null, 'PriceError -> amount must be set (can be 0).');
    assert(map['currency'] != null, 'PriceError -> currency must be set.');
    assert(map['product_token'] != null,
        'PriceError -> product_token must be set.');

    return Price(
      id: map['_id'] is ObjectId
          ? map['_id'].toHexString()
          : map['_id'].toString(),
      amount: map['amount'],
      productToken: map['product_token'],
      token: map['token'],
      currency: map['currency'],
      name: map['name'],
      active: map['active'] ?? true,
      description: map['description'],
      unitLabel: map['unit_label'],
      metadata: (map['metadata'] != null) ? Map.from(map['metadata']) : {},
      taxes: (map['taxes'] != null)
          ? List.from(map['taxes']).map((e) => e.toString()).toList()
          : [],
      tiers: (map['tiers'] != null)
          ? List.from(map['tiers']).map((e) => Tier.fromMap(e)).toList()
          : null,
      periodType: map['period_type'] != null
          ? Recurrency.values[int.tryParse(map['period_type'].toString()) ??
              Recurrency.onetime.index]
          : Recurrency.onetime,
      periodLength: map['period_length'],
      archived: map['archived'] ?? false,
      product:
          (map['product'] != null) ? Product.fromMap(map['product']) : null,
    );
  }
  factory Price.fromJson(String source) => Price.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Product(id: $id, product: $productToken, currency: $currency name: $name, description: $description, active: $active, unitLabel: $unitLabel, metadata: $metadata, taxes: $taxes, tiers: $tiers, periodType: $periodType, periodLength: $periodLength)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Price &&
        other.id == id &&
        other.productToken == productToken &&
        other.currency == currency &&
        other.name == name &&
        other.active == active &&
        other.amount == amount &&
        other.taxes == taxes &&
        other.tiers == tiers &&
        other.description == description &&
        other.unitLabel == unitLabel &&
        other.metadata == metadata &&
        other.periodType == periodType &&
        other.periodLength == periodLength;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        productToken.hashCode ^
        currency.hashCode ^
        name.hashCode ^
        description.hashCode ^
        amount.hashCode ^
        taxes.hashCode ^
        tiers.hashCode ^
        active.hashCode ^
        unitLabel.hashCode ^
        metadata.hashCode ^
        periodType.hashCode ^
        periodLength.hashCode;
  }
}

==> apollopay-1.0.4.tar.gz: lib/src/api/entities/payment/pricing/price_entity.dart <==
import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_tier_entity.dart';
import 'package:apollopay/src/api/entities/payment/product_entity.dart';
import 'package:apollopay/src/api/entities/reccurency.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Price extends Entity {
  String? id;
  String? token;
  String? productToken;
  String? currency;
  int amount;
  String? name;
  String? description;
  bool? active;
  String? unitLabel;
  Map<String, String>? metadata;
  List<String> taxes;
  List<Tier>? tiers;
  Recurrency periodType;
  int? periodLength;
  bool archived;

  // Assigned by the controller
  Product? product;

  double getPrice(double quantity) {
    if (tiers == null || tiers?.isEmpty == true) {
      return quantity * amount;
    }

    double calculatedAmount = amount.toDouble();
    double quantityLeft = quantity;

    tiers?.sort((a, b) {
      return (a.upperLimit ?? double.infinity)
          .compareTo((b.upperLimit ?? double.infinity));
    });

    for (var tier in tiers!) {
      tier.upperLimit ??= double.infinity;
      if (quantity <= (tier.upperLimit ?? 0)) {
        calculatedAmount += tier.amount + (tier.unitAmount * quantityLeft);
        quantityLeft -= tier.upperLimit ?? 0;
      }
    }

    return calculatedAmount;
  }

  Price(
      {required this.amount,
      this.currency,
      this.id,
      this.token,
      this.productToken,
      this.name,
      this.description,
      this.active,
      this.taxes = const <String>[],
      this.unitLabel,
      this.metadata,
      this.tiers,
      this.periodType = Recurrency.onetime,
      this.periodLength,
      this.archived = false,
      this.product});

  @override
  Price copyWith(
      {String? currency,
      int? amount,
      String? name,
      String? description,
      bool? active,
      String? unitLabel,
      Map<String, String>? metadata,
      List<String>? taxesTokens,
      List<Tier>? tiers,
      bool? recurring,
      int? reccurencyPerYear,
      bool? archived,
      Product? product}) {
    return Price(
        id: id,
        token: token,
        productToken: productToken,
        currency: currency ?? this.currency,
        amount: amount ?? this.amount,
        name: name ?? this.name,
        description: description ?? this.description,
        active: active ?? this.active,
        unitLabel: unitLabel ?? this.unitLabel,
        metadata: metadata ?? this.metadata,
        taxes: taxesTokens ?? taxes,
        tiers: tiers ?? this.tiers,
        periodType: periodType,
        periodLength: periodLength,
        archived: archived ?? this.archived,
        product: product ?? this.product);
  }

  @override
  Map<String, dynamic> toMap({bool databaseFormat = false}) {
    return {
      if (databaseFormat) '_id': id,
      'product_token': productToken,
      'token': token,
      'currency': currency,
      'amount': amount,
      'name': name,
      'description': description,
      'active': active,
      'unit_label': unitLabel,
      'taxes': taxes,
      'metadata': metadata,
      'tiers': tiers?.map((tier) => tier.toMap()).toList(),
      'period_type': periodType.index,
      'period_length': periodLength,
      'archived': archived,
      if (product != null) 'product': product?.toMap()
    };
  }

  factory Price.fromMap(Map<String, dynamic> map) {
    assert(map['name'] != null, 'PriceError -> name must be set.');
    assert(
        map['amount'] != null, 'PriceError -> amount must be set (can be 0).');
    assert(map['currency'] != null, 'PriceError -> currency must be set.');
    assert(map['product_token'] != null,
        'PriceError -> product_token must be set.');

    return Price(
      id: map['_id'] is ObjectId
          ? map['_id'].toHexString()
          : map['_id'].toString(),
      amount: map['amount'],
      productToken: map['product_token'],
      token: map['token'],
      currency: map['currency'],
      name: map['name'],
      active: map['active'] ?? true,
      description: map['description'],
      unitLabel: map['unit_label'],
      metadata: (map['metadata'] != null) ? Map.from(map['metadata']) : {},
      taxes: (map['taxes'] != null)
          ? List.from(map['taxes']).map((e) => e.toString()).toList()
          : [],
      tiers: (map['tiers'] != null)
          ? List.from(map['tiers']).map((e) => Tier.fromMap(e)).toList()
          : null,
      periodType: map['period_type'] != null
          ? Recurrency.values[int.tryParse(map['period_type'].toString()) ??
              Recurrency.onetime.index]
          : Recurrency.onetime,
      periodLength: map['period_length'],
      archived: map['archived'] ?? false,
      product:
          (map['product'] != null) ? Product.fromMap(map['product']) : null,
    );
  }
  factory Price.fromJson(String source) => Price.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Product(id: $id, product: $productToken, currency: $currency name: $name, description: $description, active: $active, unitLabel: $unitLabel, metadata: $metadata, taxes: $taxes, tiers: $tiers, periodType: $periodType, periodLength: $periodLength)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Price &&
        other.id == id &&
        other.productToken == productToken &&
        other.currency == currency &&
        other.name == name &&
        other.active == active &&
        other.amount == amount &&
        other.taxes == taxes &&
        other.tiers == tiers &&
        other.description == description &&
        other.unitLabel == unitLabel &&
        other.metadata == metadata &&
        other.periodType == periodType &&
        other.periodLength == periodLength;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        productToken.hashCode ^
        currency.hashCode ^
        name.hashCode ^
        description.hashCode ^
        amount.hashCode ^
        taxes.hashCode ^
        tiers.hashCode ^
        active.hashCode ^
        unitLabel.hashCode ^
        metadata.hashCode ^
        periodType.hashCode ^
        periodLength.hashCode;
  }
}
